import { CarpoolPathCreator } from '@modules/ad/core/domain/carpool-path-creator.service';
import { CarpoolPathCreatorException } from '@modules/ad/core/domain/match.errors';
import { Point } from '@modules/ad/core/domain/value-objects/point.value-object';
import { WayStep } from '@modules/ad/core/domain/value-objects/waystep.value-object';

const waypoint1: Point = new Point({
  lat: 0,
  lon: 0,
});
const waypoint2: Point = new Point({
  lat: 2,
  lon: 2,
});
const waypoint3: Point = new Point({
  lat: 5,
  lon: 5,
});
const waypoint4: Point = new Point({
  lat: 6,
  lon: 6,
});
const waypoint5: Point = new Point({
  lat: 8,
  lon: 8,
});
const waypoint6: Point = new Point({
  lat: 10,
  lon: 10,
});

describe('Carpool Path Creator Service', () => {
  it('should create a simple carpool path', () => {
    const carpoolPathCreator: CarpoolPathCreator = new CarpoolPathCreator(
      [waypoint1, waypoint6],
      [waypoint2, waypoint5],
    );
    const waysteps: WayStep[] = carpoolPathCreator.carpoolPath();
    expect(waysteps).toHaveLength(4);
    expect(waysteps[0].actors.length).toBe(1);
  });
  it('should create a simple carpool path with same destination for driver and passenger', () => {
    const carpoolPathCreator: CarpoolPathCreator = new CarpoolPathCreator(
      [waypoint1, waypoint6],
      [waypoint2, waypoint6],
    );
    const waysteps: WayStep[] = carpoolPathCreator.carpoolPath();
    expect(waysteps).toHaveLength(3);
    expect(waysteps[0].actors.length).toBe(1);
    expect(waysteps[1].actors.length).toBe(2);
    expect(waysteps[2].actors.length).toBe(2);
  });
  it('should create a simple carpool path with same waypoints for driver and passenger', () => {
    const carpoolPathCreator: CarpoolPathCreator = new CarpoolPathCreator(
      [waypoint1, waypoint6],
      [waypoint1, waypoint6],
    );
    const waysteps: WayStep[] = carpoolPathCreator.carpoolPath();
    expect(waysteps).toHaveLength(2);
    expect(waysteps[0].actors.length).toBe(2);
    expect(waysteps[1].actors.length).toBe(2);
  });
  it('should create a complex carpool path with 3 driver waypoints', () => {
    const carpoolPathCreator: CarpoolPathCreator = new CarpoolPathCreator(
      [waypoint1, waypoint3, waypoint6],
      [waypoint2, waypoint5],
    );
    const waysteps: WayStep[] = carpoolPathCreator.carpoolPath();
    expect(waysteps).toHaveLength(5);
    expect(waysteps[0].actors.length).toBe(1);
    expect(waysteps[1].actors.length).toBe(2);
    expect(waysteps[2].actors.length).toBe(1);
    expect(waysteps[3].actors.length).toBe(2);
    expect(waysteps[4].actors.length).toBe(1);
  });
  it('should create a complex carpool path with 4 driver waypoints', () => {
    const carpoolPathCreator: CarpoolPathCreator = new CarpoolPathCreator(
      [waypoint1, waypoint3, waypoint4, waypoint6],
      [waypoint2, waypoint5],
    );
    const waysteps: WayStep[] = carpoolPathCreator.carpoolPath();
    expect(waysteps).toHaveLength(6);
    expect(waysteps[0].actors.length).toBe(1);
    expect(waysteps[1].actors.length).toBe(2);
    expect(waysteps[2].actors.length).toBe(1);
    expect(waysteps[3].actors.length).toBe(1);
    expect(waysteps[4].actors.length).toBe(2);
    expect(waysteps[5].actors.length).toBe(1);
  });
  it('should create a alternate complex carpool path with 4 driver waypoints', () => {
    const carpoolPathCreator: CarpoolPathCreator = new CarpoolPathCreator(
      [waypoint1, waypoint2, waypoint5, waypoint6],
      [waypoint3, waypoint4],
    );
    const waysteps: WayStep[] = carpoolPathCreator.carpoolPath();
    expect(waysteps).toHaveLength(6);
    expect(waysteps[0].actors.length).toBe(1);
    expect(waysteps[1].actors.length).toBe(1);
    expect(waysteps[2].actors.length).toBe(2);
    expect(waysteps[3].actors.length).toBe(2);
    expect(waysteps[4].actors.length).toBe(1);
    expect(waysteps[5].actors.length).toBe(1);
  });
  it('should throw an exception if less than 2 driver waypoints are given', () => {
    try {
      new CarpoolPathCreator([waypoint1], [waypoint3, waypoint4]);
    } catch (e: any) {
      expect(e).toBeInstanceOf(CarpoolPathCreatorException);
    }
  });
  it('should throw an exception if less than 2 passenger waypoints are given', () => {
    try {
      new CarpoolPathCreator([waypoint1, waypoint6], [waypoint3]);
    } catch (e: any) {
      expect(e).toBeInstanceOf(CarpoolPathCreatorException);
    }
  });
});
